/******************************************************************************
 *
 * Module Name: atosxfwrap - include for AcpiOs* interfaces wrapping
 *                           derived from acpixf.h
 *
 *****************************************************************************/

/******************************************************************************
 *
 * 1. Copyright Notice
 *
 * Some or all of this work - Copyright (c) 1999 - 2014, Intel Corp.
 * All rights reserved.
 *
 * 2. License
 *
 * 2.1. This is your license from Intel Corp. under its intellectual property
 * rights. You may have additional license terms from the party that provided
 * you this software, covering your right to use that party's intellectual
 * property rights.
 *
 * 2.2. Intel grants, free of charge, to any person ("Licensee") obtaining a
 * copy of the source code appearing in this file ("Covered Code") an
 * irrevocable, perpetual, worldwide license under Intel's copyrights in the
 * base code distributed originally by Intel ("Original Intel Code") to copy,
 * make derivatives, distribute, use and display any portion of the Covered
 * Code in any form, with the right to sublicense such rights; and
 *
 * 2.3. Intel grants Licensee a non-exclusive and non-transferable patent
 * license (with the right to sublicense), under only those claims of Intel
 * patents that are infringed by the Original Intel Code, to make, use, sell,
 * offer to sell, and import the Covered Code and derivative works thereof
 * solely to the minimum extent necessary to exercise the above copyright
 * license, and in no event shall the patent license extend to any additions
 * to or modifications of the Original Intel Code. No other license or right
 * is granted directly or by implication, estoppel or otherwise;
 *
 * The above copyright and patent license is granted only if the following
 * conditions are met:
 *
 * 3. Conditions
 *
 * 3.1. Redistribution of Source with Rights to Further Distribute Source.
 * Redistribution of source code of any substantial portion of the Covered
 * Code or modification with rights to further distribute source must include
 * the above Copyright Notice, the above License, this list of Conditions,
 * and the following Disclaimer and Export Compliance provision. In addition,
 * Licensee must cause all Covered Code to which Licensee contributes to
 * contain a file documenting the changes Licensee made to create that Covered
 * Code and the date of any change. Licensee must include in that file the
 * documentation of any changes made by any predecessor Licensee. Licensee
 * must include a prominent statement that the modification is derived,
 * directly or indirectly, from Original Intel Code.
 *
 * 3.2. Redistribution of Source with no Rights to Further Distribute Source.
 * Redistribution of source code of any substantial portion of the Covered
 * Code or modification without rights to further distribute source must
 * include the following Disclaimer and Export Compliance provision in the
 * documentation and/or other materials provided with distribution. In
 * addition, Licensee may not authorize further sublicense of source of any
 * portion of the Covered Code, and must include terms to the effect that the
 * license from Licensee to its licensee is limited to the intellectual
 * property embodied in the software Licensee provides to its licensee, and
 * not to intellectual property embodied in modifications its licensee may
 * make.
 *
 * 3.3. Redistribution of Executable. Redistribution in executable form of any
 * substantial portion of the Covered Code or modification must reproduce the
 * above Copyright Notice, and the following Disclaimer and Export Compliance
 * provision in the documentation and/or other materials provided with the
 * distribution.
 *
 * 3.4. Intel retains all right, title, and interest in and to the Original
 * Intel Code.
 *
 * 3.5. Neither the name Intel nor any other trademark owned or controlled by
 * Intel shall be used in advertising or otherwise to promote the sale, use or
 * other dealings in products derived from or relating to the Covered Code
 * without prior written authorization from Intel.
 *
 * 4. Disclaimer and Export Compliance
 *
 * 4.1. INTEL MAKES NO WARRANTY OF ANY KIND REGARDING ANY SOFTWARE PROVIDED
 * HERE. ANY SOFTWARE ORIGINATING FROM INTEL OR DERIVED FROM INTEL SOFTWARE
 * IS PROVIDED "AS IS," AND INTEL WILL NOT PROVIDE ANY SUPPORT, ASSISTANCE,
 * INSTALLATION, TRAINING OR OTHER SERVICES. INTEL WILL NOT PROVIDE ANY
 * UPDATES, ENHANCEMENTS OR EXTENSIONS. INTEL SPECIFICALLY DISCLAIMS ANY
 * IMPLIED WARRANTIES OF MERCHANTABILITY, NONINFRINGEMENT AND FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * 4.2. IN NO EVENT SHALL INTEL HAVE ANY LIABILITY TO LICENSEE, ITS LICENSEES
 * OR ANY OTHER THIRD PARTY, FOR ANY LOST PROFITS, LOST DATA, LOSS OF USE OR
 * COSTS OF PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES, OR FOR ANY INDIRECT,
 * SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THIS AGREEMENT, UNDER ANY
 * CAUSE OF ACTION OR THEORY OF LIABILITY, AND IRRESPECTIVE OF WHETHER INTEL
 * HAS ADVANCE NOTICE OF THE POSSIBILITY OF SUCH DAMAGES. THESE LIMITATIONS
 * SHALL APPLY NOTWITHSTANDING THE FAILURE OF THE ESSENTIAL PURPOSE OF ANY
 * LIMITED REMEDY.
 *
 * 4.3. Licensee shall not export, either directly or indirectly, any of this
 * software or system incorporating such software without first obtaining any
 * required license or other approval from the U. S. Department of Commerce or
 * any other agency or department of the United States Government. In the
 * event Licensee exports any such software from the United States or
 * re-exports any such software from a foreign destination, Licensee shall
 * ensure that the distribution and export/re-export of the software is in
 * compliance with all laws, regulations, orders, or other restrictions of the
 * U.S. Export Administration Regulations. Licensee agrees that neither it nor
 * any of its subsidiaries will export/re-export any technical data, process,
 * software, or service, directly or indirectly, to any country for which the
 * United States government or any agency thereof requires an export license,
 * other governmental approval, or letter of assurance, without first obtaining
 * such license, approval or letter.
 *
 *****************************************************************************/

#ifndef _AT_OSXF_WRAP
#define _AT_OSXF_WRAP

/*
 * OSL Initialization and shutdown primitives
 */
ACPI_STATUS
AcpiOsActualInitialize (
    void);

ACPI_STATUS
AcpiOsActualTerminate (
    void);


/*
 * ACPI Table interfaces
 */
/*
ACPI_STATUS
AcpiOsActualGetRootPointer (
    UINT32                  Flags,
    ACPI_POINTER            *Address);
*/
ACPI_PHYSICAL_ADDRESS
AcpiOsActualGetRootPointer (
    void);

ACPI_STATUS
AcpiOsActualPredefinedOverride (
    const ACPI_PREDEFINED_NAMES *InitVal,
    ACPI_STRING                 *NewVal);

ACPI_STATUS
AcpiOsActualTableOverride (
    ACPI_TABLE_HEADER       *ExistingTable,
    ACPI_TABLE_HEADER       **NewTable);


/*
 * Synchronization primitives
 */
ACPI_STATUS
AcpiOsActualCreateSemaphore (
    UINT32                  MaxUnits,
    UINT32                  InitialUnits,
    ACPI_HANDLE             *OutHandle);

ACPI_STATUS
AcpiOsActualDeleteSemaphore (
    ACPI_HANDLE             Handle);

ACPI_STATUS
AcpiOsActualWaitSemaphore (
    ACPI_HANDLE             Handle,
    UINT32                  Units,
    UINT16                  Timeout);

ACPI_STATUS
AcpiOsActualSignalSemaphore (
    ACPI_HANDLE             Handle,
    UINT32                  Units);

ACPI_STATUS
AcpiOsActualCreateLock (
    ACPI_HANDLE             *OutHandle);

void
AcpiOsActualDeleteLock (
    ACPI_HANDLE             Handle);

ACPI_CPU_FLAGS
AcpiOsActualAcquireLock (
    ACPI_HANDLE             Handle);

void
AcpiOsActualReleaseLock (
    ACPI_HANDLE             Handle,
    ACPI_CPU_FLAGS          Flags);


/*
 * Memory allocation and mapping
 */
void *
AcpiOsActualAllocate (
    ACPI_SIZE               Size);

void
AcpiOsActualFree (
    void *                  Memory);

/*
ACPI_STATUS
AcpiOsActualMapMemory (
    ACPI_PHYSICAL_ADDRESS   PhysicalAddress,
    ACPI_SIZE               Size,
    void                    **LogicalAddress);
*/
void *
AcpiOsActualMapMemory (
    ACPI_PHYSICAL_ADDRESS   Where,
    ACPI_SIZE               Length);

void
AcpiOsActualUnmapMemory (
    void                    *LogicalAddress,
    ACPI_SIZE               Size);

ACPI_STATUS
AcpiOsActualGetPhysicalAddress (
    void                    *LogicalAddress,
    ACPI_PHYSICAL_ADDRESS   *PhysicalAddress);


/*
 * Memory/Object Cache
 */
ACPI_STATUS
AcpiOsActualCreateCache (
    char                    *CacheName,
    UINT16                  ObjectSize,
    UINT16                  MaxDepth,
    ACPI_CACHE_T            **ReturnCache);

ACPI_STATUS
AcpiOsActualDeleteCache (
    ACPI_CACHE_T            *Cache);

ACPI_STATUS
AcpiOsActualPurgeCache (
    ACPI_CACHE_T            *Cache);

void *
AcpiOsActualAcquireObject (
    ACPI_CACHE_T            *Cache);

ACPI_STATUS
AcpiOsActualReleaseObject (
    ACPI_CACHE_T            *Cache,
    void                    *Object);


/*
 * Interrupt handlers
 */
ACPI_STATUS
AcpiOsActualInstallInterruptHandler (
    UINT32                  InterruptNumber,
    ACPI_OSD_HANDLER        ServiceRoutine,
    void                    *Context);

ACPI_STATUS
AcpiOsActualRemoveInterruptHandler (
    UINT32                  InterruptNumber,
    ACPI_OSD_HANDLER        ServiceRoutine);


/*
 * Threads and Scheduling
 */
ACPI_THREAD_ID
AcpiOsActualGetThreadId (
    void);

ACPI_STATUS
AcpiOsActualExecute (
    ACPI_EXECUTE_TYPE       Type,
    ACPI_OSD_EXEC_CALLBACK  Function,
    void                    *Context);

void
AcpiOsActualWaitEventsComplete (
    void                    *Context);

void
AcpiOsActualSleep (
    ACPI_INTEGER            Milliseconds);

void
AcpiOsActualStall (
    UINT32                  Microseconds);

ACPI_STATUS
AcpiOsActualValidateInterface (
    char                    *Interface);

ACPI_STATUS
AcpiOsActualValidateAddress (
    UINT8                   SpaceId,
    ACPI_PHYSICAL_ADDRESS   Address,
    ACPI_SIZE               Length);

/*
 * Platform and hardware-independent I/O interfaces
 */
ACPI_STATUS
AcpiOsActualReadPort (
    ACPI_IO_ADDRESS         Address,
    UINT32                  *Value,
    UINT32                  Width);

ACPI_STATUS
AcpiOsActualWritePort (
    ACPI_IO_ADDRESS         Address,
    UINT32                  Value,
    UINT32                  Width);


/*
 * Platform and hardware-independent physical memory interfaces
 */
ACPI_STATUS
AcpiOsActualReadMemory (
    ACPI_PHYSICAL_ADDRESS   Address,
    UINT64                  *Value,
    UINT32                  Width);

ACPI_STATUS
AcpiOsActualWriteMemory (
    ACPI_PHYSICAL_ADDRESS   Address,
    UINT64                  Value,
    UINT32                  Width);


/*
 * Platform and hardware-independent PCI configuration space access
 * Note: Can't use "Register" as a parameter, changed to "Reg" --
 * certain compilers complain.
 */
ACPI_STATUS
AcpiOsActualReadPciConfiguration (
    ACPI_PCI_ID             *PciId,
    UINT32                  Reg,
    void                    *Value,
    UINT32                  Width);

ACPI_STATUS
AcpiOsActualWritePciConfiguration (
    ACPI_PCI_ID             *PciId,
    UINT32                  Reg,
    ACPI_INTEGER            Value,
    UINT32                  Width);

/*
 * Interim function needed for PCI IRQ routing
 */
void
AcpiOsActualDerivePciId(
    ACPI_HANDLE             Rhandle,
    ACPI_HANDLE             Chandle,
    ACPI_PCI_ID             **PciId);

/*
 * Miscellaneous
 */
BOOLEAN
AcpiOsActualReadable (
    void                    *Pointer,
    ACPI_SIZE               Length);

BOOLEAN
AcpiOsActualWritable (
    void                    *Pointer,
    ACPI_SIZE               Length);

UINT64
AcpiOsActualGetTimer (
    void);

ACPI_STATUS
AcpiOsActualSignal (
    UINT32                  Function,
    void                    *Info);

/*
 * Debug print routines
 */
void ACPI_INTERNAL_VAR_XFACE
AcpiOsActualPrintf (
    const char              *Format,
    ...);

void
AcpiOsActualVprintf (
    const char              *Format,
    va_list                 Args);

void
AcpiOsActualRedirectOutput (
    void                    *Destination);


/*
 * Debug input
 */
UINT32
AcpiOsActualGetLine (
    char                    *Buffer);


/*
 * Directory manipulation
 */
void *
AcpiOsActualOpenDirectory (
    char                    *Pathname,
    char                    *WildcardSpec,
    char                    RequestedFileType);

/* RequesteFileType values */

#define REQUEST_FILE_ONLY                   0
#define REQUEST_DIR_ONLY                    1


char *
AcpiOsActualGetNextFilename (
    void                    *DirHandle);

void
AcpiOsActualCloseDirectory (
    void                    *DirHandle);

/*
 * Debug
 */
void
AcpiOsActualDbgAssert(
    void                    *FailedAssertion,
    void                    *FileName,
    UINT32                  LineNumber,
    char                    *Message);

#endif /* _AT_OSXF_WRAP */
